@@ -156,7 +156,6 @@ class WaldurClient(object):
         MarketplaceCategories = "marketplace-categories"
         MarketplaceProviderOffering = "marketplace-provider-offerings"
         MarketplaceOrder = "marketplace-orders"
-        MarketplaceOrderItem = "marketplace-order-items"
         MarketplaceProviderPlan = "marketplace-plans"
         MarketplacePublicOffering = "marketplace-public-offerings"
         MarketplaceResources = "marketplace-resources"
@@ -183,6 +182,7 @@ class WaldurClient(object):
         TenantSecurityGroup = "openstack-security-groups"
         UserInvitations = "user-invitations"
         Users = "users"
+        Roles = "roles"
         Volume = "openstacktenant-volumes"
         VolumeType = "openstacktenant-volume-types"
         ServerGroup = "openstacktenant-server-groups"
@@ -219,6 +219,8 @@ class WaldurClient(object):
     def _build_resource_url(
         self, endpoint, uid1, action=None, sub_endpoint=None, uid2=None
     ):
+        if not uid1:
+            raise WaldurClientException("Resource ID is empty.")
         parts = [endpoint, str(uid1)]
 
         if sub_endpoint is not None and uid2 is not None:
@@ -470,9 +472,9 @@ class WaldurClient(object):
         url = self._build_url(self.Endpoints.Users)
         return self._get_count(url, **kwargs)
 
-    def count_customer_permissions(self, **kwargs):
-        url = self._build_url(self.Endpoints.CustomerPermissions)
-        return self._get_count(url, **kwargs)
+    def get_roles(self, **kwargs):
+        url = self._build_url(self.Endpoints.Roles)
+        return self._get_all(url, **kwargs)
 
     def list_ssh_keys(self):
         url = self._build_url(self.Endpoints.SshKey)
@@ -824,7 +826,7 @@ class WaldurClient(object):
         if offering_uuid is not None:
             params["offering_uuid"] = offering_uuid
         if fields is not None:
-            if type(fields) is not list:
+            if not isinstance(fields, list):
                 fields = [fields]
             params["field"] = fields
 
@@ -1146,68 +1148,76 @@ class WaldurClient(object):
     def list_orders(self, filters=None):
         return self._query_resource_list(self.Endpoints.MarketplaceOrder, filters)
 
-    def get_order_item(self, order_item_uuid):
-        return self._get_resource(
-            WaldurClient.Endpoints.MarketplaceOrderItem, order_item_uuid
-        )
-
-    def list_order_items(self, filters=None):
-        return self._query_resource_list(self.Endpoints.MarketplaceOrderItem, filters)
-
-    def marketplace_order_item_approve(self, order_item_uuid: str):
+    def marketplace_order_approve_by_consumer(self, order_uuid: str):
         url = self._build_resource_url(
-            self.Endpoints.MarketplaceOrderItem,
-            order_item_uuid,
-            action="approve",
+            self.Endpoints.MarketplaceOrder,
+            order_uuid,
+            action="approve_by_consumer",
         )
         return self._post(url, valid_states=[200])
 
-    def marketplace_order_item_reject(self, order_item_uuid: str):
+    def marketplace_order_approve_by_provider(self, order_uuid: str):
         url = self._build_resource_url(
-            self.Endpoints.MarketplaceOrderItem,
-            order_item_uuid,
-            action="reject",
+            self.Endpoints.MarketplaceOrder,
+            order_uuid,
+            action="approve_by_provider",
         )
         return self._post(url, valid_states=[200])
 
-    def marketplace_order_item_terminate(self, order_item_uuid: str):
+    def marketplace_order_reject_by_consumer(self, order_uuid: str):
         url = self._build_resource_url(
-            self.Endpoints.MarketplaceOrderItem,
-            order_item_uuid,
-            action="terminate",
+            self.Endpoints.MarketplaceOrder,
+            order_uuid,
+            action="reject_by_consumer",
+        )
+        return self._post(url, valid_states=[200])
+
+    def marketplace_order_reject_by_provider(self, order_uuid: str):
+        url = self._build_resource_url(
+            self.Endpoints.MarketplaceOrder,
+            order_uuid,
+            action="reject_by_provider",
+        )
+        return self._post(url, valid_states=[200])
+
+    def marketplace_order_cancel(self, order_uuid: str):
+        url = self._build_resource_url(
+            self.Endpoints.MarketplaceOrder,
+            order_uuid,
+            action="cancel",
         )
 
         return self._post(url, valid_states=[202])
 
-    def marketplace_order_item_set_state_executing(self, order_item_uuid: str):
+    def marketplace_order_set_state_executing(self, order_uuid: str):
         url = self._build_resource_url(
-            self.Endpoints.MarketplaceOrderItem,
-            order_item_uuid,
+            self.Endpoints.MarketplaceOrder,
+            order_uuid,
             action="set_state_executing",
         )
 
         return self._post(url, valid_states=[200])
 
-    def marketplace_order_item_set_state_done(self, order_item_uuid: str):
+    def marketplace_order_set_state_done(self, order_uuid: str):
         url = self._build_resource_url(
-            self.Endpoints.MarketplaceOrderItem,
-            order_item_uuid,
+            self.Endpoints.MarketplaceOrder,
+            order_uuid,
             action="set_state_done",
         )
 
         return self._post(url, valid_states=[200])
 
-    def marketplace_order_item_set_state_erred(
+    def marketplace_order_set_state_erred(
         self,
-        order_item_uuid: str,
+        order_uuid: str,
         error_message: str = "",
         error_traceback: str = "",
     ):
         payload = {"error_message": error_message, "error_traceback": error_traceback}
 
         url = self._build_resource_url(
-            self.Endpoints.MarketplaceOrderItem,
-            order_item_uuid,
+            self.Endpoints.MarketplaceOrder,
+            order_uuid,
             action="set_state_erred",
         )
 
@@ -1223,10 +1233,10 @@ class WaldurClient(object):
         waited = 0
         while True:
             order = self.get_order(order_uuid)
-            if order["items"][0]["state"] == "erred":
-                raise InvalidStateError(order["items"][0]["error_message"])
+            if order["state"] == "erred":
+                raise InvalidStateError(order["error_message"])
 
-            resource_uuid = order["items"][0].get(resource_field)
+            resource_uuid = order.get(resource_field)
             if resource_uuid:
                 return resource_uuid
             time.sleep(interval)
@@ -1663,16 +1673,19 @@ class WaldurClient(object):
     ):
         attributes = attributes or {}
         limits = limits or {}
-        order_item = {
+        payload = {
+            "project": self._build_resource_url(self.Endpoints.Project, project_uuid),
             "offering": self._build_resource_url(
                 self.Endpoints.MarketplacePublicOffering, offering_uuid
             ),
             "attributes": attributes,
             "limits": limits,
+            # TODO: replace with checkbox data from frontend
+            "accepting_terms_of_service": True,
         }
 
         if plan_uuid:
-            order_item["plan"] = self._build_resource_url(
+            payload["plan"] = self._build_resource_url(
                 self.Endpoints.MarketplacePublicOffering,
                 offering_uuid,
                 sub_endpoint="plans",
@@ -1680,15 +1693,8 @@ class WaldurClient(object):
             )
 
         if callback_url:
-            order_item["callback_url"] = callback_url
+            payload["callback_url"] = callback_url
 
-        # TODO: replace with checkbox data from frontend
-        order_item["accepting_terms_of_service"] = True
-
-        payload = {
-            "project": self._build_resource_url(self.Endpoints.Project, project_uuid),
-            "items": [order_item],
-        }
         return self._create_resource(self.Endpoints.MarketplaceOrder, payload=payload)
 
     def marketplace_resource_update_limits_order(
